unit unitMainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, jpeg, ExtCtrls, JvHidControllerClass, StdCtrls,Math,ShellAPI;

type

  TAxisData = record
    X: Byte;
    Y: Byte;
    Z: Byte;
  end;

  TIntAxisData = record
    X: Integer;
    Y: Integer;
    Z: Integer;
    Unknown1: Integer;
    Unknown2: Integer;
  end;


  TMainForm = class(TForm)
    Image1: TImage;
    HidCtl: TJvHidDeviceController;
    ListBox1: TListBox;
    InitButton: TButton;
    SetMasterButton: TButton;
    DrawTimer: TTimer;
    imgGraph: TImage;
    LabelX: TLabel;
    LabelY: TLabel;
    LabelZ: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    LabelURL: TLabel;
    procedure HidCtlDeviceChange(Sender: TObject);
    procedure HidCtlDeviceData(HidDev: TJvHidDevice; ReportID: Byte;
      const Data: Pointer; Size: Word);
    procedure InitButtonClick(Sender: TObject);
    procedure SetMasterButtonClick(Sender: TObject);
    procedure DrawTimerTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure LabelURLClick(Sender: TObject);
  private
    { Private 錾 }
    CurrentDevice: TJvHidDevice;
    bmpGraph: TBitmap;
    GraphData: array [0..1023] of TIntAxisData;
    procedure SendToDevice(Buf: array of Byte; WriteLength: Integer);
    procedure AnalyzePacket(ReportID: Integer; ReportData: PChar;
      Size: Word);
    procedure DrawImage;
    procedure PushXYZ(Data: PChar);
  public
    { Public 錾 }
  end;

var
  MainForm: TMainForm;
const
  SIXAXIS_VID = $054c;
  SIXAXIS_PID = $0268;
implementation

{$R *.dfm}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  MainForm.DoubleBuffered := True;
  bmpGraph := TBitmap.Create;
  bmpGraph.Width := imgGraph.Width;
  bmpGraph.Height := imgGraph.Height;
end;

procedure TMainForm.HidCtlDeviceChange(Sender: TObject);
begin
  if (CurrentDevice = nil) and HidCtl.CheckOutByID(CurrentDevice, SIXAXIS_VID, SIXAXIS_PID) then
  begin
  ListBox1.Items.Add('sixaxis connected.');
  end
  else
  begin
  ListBox1.Items.Add('sixaxis removed.');
  end;

end;
procedure TMainForm.SendToDevice(Buf: array of Byte; WriteLength: Integer);
var
  numToWrite: Cardinal;
  numWritten: Cardinal;
  numErr: DWORD;
  Str: string;
  i:integer;
begin
  if not Assigned(CurrentDevice) then
  begin
    MainForm.Caption := 'Device not found';
    Exit;
  end;

  numToWrite := CurrentDevice.Caps.OutputReportByteLength;

  CurrentDevice.OpenFile;
  if not CurrentDevice.WriteFile(Buf, numToWrite, numWritten) then
  begin
    numErr := GetLastError;
    MainForm.Caption := 'Error(' + IntToStr(numErr) + '):' + SysErrorMessage(numErr);
    ListBox1.Items.Add(SysErrorMessage(numErr));
  end
  else
  begin
    CurrentDevice.CloseFile;
      for I := 0 to numToWrite - 1 do
        Str := Str + Format('%.2x ', [Buf[I]]);
      ListBox1.Items.Add('write '+ str);
  end;
end;

procedure TMainForm.HidCtlDeviceData(HidDev: TJvHidDevice; ReportID: Byte;
  const Data: Pointer; Size: Word);
var
  i:integer;
  Str: string;
  Str2: string;
begin
  AnalyzePacket(ReportID, PChar(Data), Size);

  Str := Format('R %.2x ', [ReportID]);
  Str2 := 'No   ';

  for I := 0 to Size - 1 do
  begin
    Str2 := Str2 + Format('%.2d ', [Cardinal(I)]);
    Str := Str + Format('%.2x ', [Cardinal(PChar(Data)[I])]);
  end;
  ListBox1.Items.insert(0,str2);//index
  ListBox1.Items.insert(0,str);

  while(ListBox1.Items.Count  > 100)
  do
    ListBox1.Items.Delete(ListBox1.Items.Count-1);
  begin
  end;

end;


procedure TMainForm.AnalyzePacket(ReportID: Integer; ReportData: PChar; Size: Word);
begin
PushXYZ(ReportData);
end;
procedure TMainForm.PushXYZ(Data: PChar);
var
  I: Integer;
begin
  for i := High(GraphData) downto 1 do
    GraphData[I] := GraphData[I - 1];

  GraphData[0].Unknown1 := (Byte(Data[4]))+10;//what's this? equals Data[41]
  GraphData[0].Unknown2 := (Byte(Data[5]))+10;//what's this? equals Data[45]

  GraphData[0].X := (Byte(Data[40])-1)*256+Byte(Data[41]);
  GraphData[0].Y := (Byte(Data[42])-1)*256+Byte(Data[43]);
  GraphData[0].Z := (Byte(Data[44])-1)*256+Byte(Data[45]);

  labelX.caption:=inttostr(GraphData[0].X) ;
  labelY.caption:=inttostr(GraphData[0].Y) ;
  labelZ.caption:=inttostr(GraphData[0].Z) ;
end;

//don't work. matter of endian?
procedure TMainForm.InitButtonClick(Sender: TObject);
var
  i:integer;
  Buf: array[0..64] of Byte;
begin
  //initialize buffer
  for I := 0 to 64 do  Buf[i]:= 0;

  Buf[0] := $01;
  Buf[1] := $53;
  Buf[2] := $f4;
  Buf[3] := $42;
  Buf[4] := $03;
  Buf[5] := $00;
  Buf[6] := $00;
  SendToDevice(Buf, 6);

//static void enable_sixaxis(int sock){
//  char msg[] = { 0x53 /*HIDP_TRANS_SET_REPORT | HIDP_DATA_RTYPE_FEATURE*/,
//		 0xf4,  0x42, 0x03, 0x00, 0x00 };
//  send(sock, msg, sizeof(msg), 0);
//}

end;

procedure TMainForm.SetMasterButtonClick(Sender: TObject);
var
  i:integer;
  Buf: array[0..64] of Byte;
  Mac: array[0..5] of Byte;
begin
  //initialize buffer
  for I := 0 to 64  do  Buf[i]:= 0;

  Mac[0]:=$00; //set here the bluetooth address
  Mac[1]:=$00;
  Mac[2]:=$00;
  Mac[3]:=$00;
  Mac[4]:=$00;
  Mac[5]:=$00;


  Buf[0] := $01;
  Buf[1] := $00;
  Buf[2] := Mac[0];
  Buf[3] := Mac[1];
  Buf[4] := Mac[2];
  Buf[5] := Mac[3];
  Buf[6] := Mac[4];
  Buf[7] := Mac[5];
  SendToDevice(Buf, 7);

//void set_master(usb_dev_handle *devh, int itfnum, int mac[6]) {
//  printf("Setting master bd_addr to %02x:%02x:%02x:%02x:%02x:%02x\n",
//	 mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
//  char msg[8]= { 0x01, 0x00, mac[0],mac[1],mac[2],mac[3],mac[4],mac[5] };
//  int res = usb_control_msg
//    (devh,
//     USB_DIR_OUT | USB_TYPE_CLASS | USB_RECIP_INTERFACE,
//     0x09,
//     0x03f5, itfnum, msg, sizeof(msg),
//     5000);
//  if ( res < 0 ) fatal("USB_REQ_SET_CONFIGURATION");
//}
end;

procedure TMainForm.DrawTimerTimer(Sender: TObject);
begin
DrawImage();
end;

procedure TMainForm.DrawImage();
var
  i: Integer;
  Count:integer;
begin

  bmpGraph.Canvas.FillRect(bmpGraph.Canvas.ClipRect);

  bmpGraph.Canvas.Pen.Color := clBlue;
  bmpGraph.Canvas.MoveTo(0, GraphData[0].X );
  Count:=Min(bmpGraph.Width, 1024);
  for i := 0 to Count do
    bmpGraph.Canvas.LineTo(i, GraphData[i].X);

  bmpGraph.Canvas.MoveTo(0, GraphData[0].Y);
  bmpGraph.Canvas.Pen.Color := clRed;
  for i := 0 to Count do
    bmpGraph.Canvas.LineTo(i, GraphData[i].Y);

  bmpGraph.Canvas.MoveTo(0, GraphData[0].Z );
  bmpGraph.Canvas.Pen.Color := clLime;
  for i := 0 to Count do
    bmpGraph.Canvas.LineTo(i, GraphData[i].Z);

{
  bmpGraph.Canvas.MoveTo(0, GraphData[0].Unknown1 );
  bmpGraph.Canvas.Pen.Color := clSkyBlue;
  for i := 0 to Count do
    bmpGraph.Canvas.LineTo(i, GraphData[i].Unknown1);


  bmpGraph.Canvas.MoveTo(0, GraphData[0].Unknown2 );
  bmpGraph.Canvas.Pen.Color := clPurple;
  for i := 0 to Count do
    bmpGraph.Canvas.LineTo(i, GraphData[i].Unknown2);
}

//  BitBlt(imgGraph.Picture.Bitmap.Canvas.Handle,0,0,bmpGraph.width,bmpGraph.height,bmpGraph.Canvas.Handle,0,0,SRCCOPY);
  imgGraph.Picture.Bitmap.Assign(bmpGraph);

end;

procedure TMainForm.LabelURLClick(Sender: TObject);
begin
  ShellExecute(Handle, 'open', PChar('http://onakasuita.org/ps3/'), '',PChar(ExtractFilePath(Application.ExeName)),SW_SHOW);
end;

end.
